#######################################################################
#
#  MerlinSkinThemes for Dreambox/Enigma-2
#  Coded by marthom (c)2012, 2013
#
#  Support: www.dreambox-tools.info
#  E-Mail: marthom@dreambox-tools.info
#
#  This plugin is open source but it is NOT free software.
#
#  This plugin may only be distributed to and executed on hardware which
#  is licensed by Dream Multimedia GmbH.
#  In other words:
#  It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
#  to hardware which is NOT licensed by Dream Multimedia GmbH.
#  It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
#  on hardware which is NOT licensed by Dream Multimedia GmbH.
#
#  If you want to use or modify the code or parts of it,
#  you have to keep MY license and inform me about the modifications by mail.
#
#######################################################################

from Plugins.Plugin import PluginDescriptor

from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.Standby import TryQuitMainloop
from Screens.ChoiceBox import ChoiceBox
from Screens.Console import Console

from skin import parseColor

from Components.ActionMap import ActionMap, HelpableActionMap
from Components.Button import Button
from Components.Input import Input
from Components.Label import Label
from Components.Sources.List import List
from Components.MenuList import MenuList
from Components.config import config, configfile, ConfigYesNo, ConfigSubsection, getConfigListEntry, ConfigSelection, ConfigNumber, ConfigText, ConfigInteger
from Components.ConfigList import ConfigListScreen

from enigma import eListbox, eListboxPythonMultiContent, gFont, RT_HALIGN_LEFT, RT_HALIGN_CENTER, RT_HALIGN_RIGHT, RT_VALIGN_CENTER

from Tools.Directories import resolveFilename, SCOPE_SKIN, SCOPE_CURRENT_SKIN, fileExists
from Tools.Notifications import AddPopup

import xml.etree.cElementTree as Tree
from xml.dom import minidom

import socket
import struct
import base64
import shutil
import os

PluginVersion = "v1.6.3"
SkinXML = config.skin.primary_skin.value
SkinFile = resolveFilename(SCOPE_SKIN) + SkinXML
SkinName = SkinXML[0:SkinXML.find("/")]
ThemeFile = resolveFilename(SCOPE_SKIN) + SkinName + "/themes.xml"

config.plugins.MerlinSkinThemes = ConfigSubsection()
config.plugins.MerlinSkinThemes.Skin = ConfigText(default=SkinName)
config.plugins.MerlinSkinThemes.selSkin = ConfigText(default=SkinName)
config.plugins.MerlinSkinThemes.ColorTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.FontTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.BorderSetTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme = ConfigText(default="")
config.plugins.MerlinSkinThemes.InfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.ChannelSelection = ConfigText(default="")
config.plugins.MerlinSkinThemes.MovieSelection = ConfigText(default="")
config.plugins.MerlinSkinThemes.MoviePlayer = ConfigText(default="")
config.plugins.MerlinSkinThemes.SecondInfoBar = ConfigText(default="")
config.plugins.MerlinSkinThemes.MessageBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.InputBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.ChoiceBox = ConfigText(default="")
config.plugins.MerlinSkinThemes.PreviewScreen = ConfigText(default="")

import MerlinPreviewScreen
import MerlinSkinInfo

class MerlinSkinThemes(Screen, HelpableScreen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="1110,550" title="%s" >
			<widget name="ListLabel" position="10,10" size="145,30" font="Regular;17" zPosition="2" valign="center" halign="left" />
			<widget name="SkinsList" position="10,40" size="575,400" scrollbarMode="showOnDemand" zPosition="1" />

			<widget name="config" position="10,40" size="575,450" scrollbarMode="showOnDemand" zPosition="1" /> 
			
			<widget name="key_red" position="10,500" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_green" position="155,500" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_yellow" position="300,500" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />
			<widget name="key_blue" position="445,500" size="140,40" valign="center" halign="center" zPosition="3" transparent="1" font="Regular;18" />

			<ePixmap name="red" position="10,500" zPosition="1" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="blend" />
			<ePixmap name="green" position="155,500" zPosition="1" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="blend" />
			<ePixmap name="yellow" position="300,500" zPosition="1" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="blend" />
			<ePixmap name="blue" position="445,500" zPosition="1" size="140,40" pixmap="skin_default/buttons/blue.png" transparent="1" alphatest="blend" />

			<widget name="DescLabel" position="600,10" size="145,30" font="Regular;17" zPosition="2" valign="center" halign="left" />
			<widget name="DescText" position="600,40" size="500,400" font="Regular;20" zPosition="2" halign="left" />
		</screen>"""% _("MerlinSkinThemes")

	ThemeName = ""
	selSkinName = ""
	selThemeFile = ""
	selColorThemeName = ""
	selFontThemeName = ""
	selScreenThemeName = ""
	selScreenName = ""
	
	def __init__(self, session):
		print "[MST] running..."
		
		config.plugins.MerlinSkinThemes.PreviewScreen.value = "InfoBar"
		config.plugins.MerlinSkinThemes.PreviewScreen.save()		
		
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.clist = []
		ConfigListScreen.__init__(self, self.clist)
		
		self.setTitle("MerlinSkinThemes " + PluginVersion + " - (c) marthom 2013 - www.dreambox-tools.info")

		self["ListLabel"] = Label(_("Skinlist"))
		self["DescLabel"] = Label(_("Description"))
		self["DescText"] = Label(_("[OK] = switch to theme config screen for selected skin\n\n[create themes] and [delete] takes some time - please wait"))

		self.curList = "SkinsList"
		
		self["key_red"] = Button(_("exit"))
		self["key_green"] = Button(_("switch to skin"))
		self["key_yellow"] = Button(_("save as design"))
		self["key_blue"] = Button(_("skin info"))
		
		self["SkinsList"] = GetSkinsList([])

		self.onSelectionChanged = [ ]
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":     (self.exit, _("Close plugin")),
			"green":   self.buttonGreen,
			"yellow":  self.buttonYellow,
			"blue":    self.buttonBlue,
		}, -1)
		
		self["DirectionActions"] = HelpableActionMap(self, "DirectionActions",
		{
			"up":		(self.up, _("Move cursor up")),
			"down":		(self.down, _("Move cursor down")),
			"left":		(self.left, _("Move cursor left")),
			"right":	(self.right, _("Move cursor right")),
		}, -1)

		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.ok, _("")),
			"cancel":	(self.exit, _("Close plugin")),
		}, -1)

		self.updateSkinsList()
		
		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		self.onLayoutFinish.append(self.startRun)

	def startRun(self):
		self["SkinsList"].onSelectionChanged.append(self.changedSkinsList)

		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		self["config"].hide()
		
		self["SkinsList"].up()
		listend = self["SkinsList"].getSelectionIndex()
		self["SkinsList"].moveToIndex(0)
		activeItem = 0
		for index in range(listend):
			self["SkinsList"].moveToIndex(index)
			if self["SkinsList"].getCurrent()[1][7] == SkinName:
				activeItem = index
				
		self["SkinsList"].moveToIndex(activeItem)
		

	def readThemes(self):
		#print "[MST] selThemeFile: %s" % MerlinSkinThemes.selThemeFile

		defaultdesign = None
		defaultcolortheme = None
		defaultfonttheme = None
		defaultbordersettheme = None
		defaultwindowstylescrollbartheme = None
		defaultinfobar = None
		defaultchannelselection = None
		defaultmovieselection = None
		defaultmovieplayer = None
		defaultsecondinfobar = None
		defaultmessagebox = None
		defaultinputbox = None
		defaultchoicebox = None
		
		self.clist = []

		try:
			xml = Tree.parse(MerlinSkinThemes.selThemeFile)

			selSkinList = []
			selSkinList.append(MerlinSkinThemes.selSkinName)
			config.plugins.MerlinSkinThemes.selSkin = ConfigSelection(default=MerlinSkinThemes.selSkinName, choices = selSkinList)
			self.clist.append(getConfigListEntry("Skin", config.plugins.MerlinSkinThemes.selSkin))
			
			self.clist.append(getConfigListEntry(_("----- PREVIEW ----------------------------------------------------------------"), ))
				
			PreviewList = []
			PreviewList.append("InfoBar")
			PreviewList.append("ChannelSelection")
			PreviewList.append("MovieSelection")
			PreviewList.append("SecondInfoBar")
			#PreviewList.append("MessageBox")
			#PreviewList.append("InputBox")
			config.plugins.MerlinSkinThemes.PreviewScreen = ConfigSelection(default=PreviewList[0], choices = PreviewList)
			self.clist.append(getConfigListEntry("PreviewScreen", config.plugins.MerlinSkinThemes.PreviewScreen))

			self.clist.append(getConfigListEntry(_("----- DESIGNS -----------------------------------------------------------------"), ))

			# Designs
			DesignList = []
			if xml.find("designs") is not None:
				DesignList.append("-none-")
				ds = xml.find("designs")
				for design in ds.findall("design"):
					DesignList.append(design.get("name"))
					#if design.get("value") == "active":
					#	defaultdesign = design.get("name")
					defaultdesign = "-none-"

			if len(DesignList) > 0:
				self.MSTDesigns = ConfigSelection(default=defaultdesign, choices = DesignList)
				config.plugins.MerlinSkinThemes.Design = self.MSTDesigns
				self.clist.append(getConfigListEntry("Design", self.MSTDesigns))
			else:
				self.MSTDesigns = ConfigSelection(default=defaultdesign, choices = ["-none-"])
				config.plugins.MerlinSkinThemes.Design = self.MSTDesigns
				self.clist.append(getConfigListEntry("Design", self.MSTDesigns))

			self.clist.append(getConfigListEntry(_("----- THEMES -----------------------------------------------------------------"), ))

			# COLOR
			ColorThemeList = []
			if xml.find("colortheme") is not None:
				for theme in xml.findall("colortheme"):
					ColorThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultcolortheme = theme.get("name")

			if len(ColorThemeList) > 0:
				self.TColor = ConfigSelection(default=defaultcolortheme, choices = ColorThemeList)
				config.plugins.MerlinSkinThemes.ColorTheme = self.TColor
				self.clist.append(getConfigListEntry("ColorTheme", self.TColor))

			# FONT
			FontThemeList = []
			if xml.find("fonttheme") is not None:
				for theme in xml.findall("fonttheme"):
					FontThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultfonttheme = theme.get("name")

			if len(FontThemeList) > 0:
				self.TFont = ConfigSelection(default=defaultfonttheme, choices = FontThemeList)
				config.plugins.MerlinSkinThemes.FontTheme = self.TFont
				self.clist.append(getConfigListEntry("FontTheme", self.TFont))

			# BORDERSET
			BorderSetThemeList = []
			if xml.find("bordersettheme") is not None:
				for theme in xml.findall("bordersettheme"):
					BorderSetThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultbordersettheme = theme.get("name")

			if len(BorderSetThemeList) > 0:
				self.TBorder = ConfigSelection(default=defaultbordersettheme, choices = BorderSetThemeList)
				config.plugins.MerlinSkinThemes.BorderSetTheme = self.TBorder
				self.clist.append(getConfigListEntry("BorderSetTheme", self.TBorder))
				
			# WINDOSTYLESCROLLBAR
			WindowStyleScrollbarThemeList = []
			if xml.find("windowstylescrollbartheme") is not None:
				for theme in xml.findall("windowstylescrollbartheme"):
					WindowStyleScrollbarThemeList.append(theme.get("name"))
					if theme.get("value") == "active":
						defaultwindowstylescrollbartheme = theme.get("name")

			if len(WindowStyleScrollbarThemeList) > 0:
				self.TScroll = ConfigSelection(default=defaultwindowstylescrollbartheme, choices = WindowStyleScrollbarThemeList)
				config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme = self.TScroll
				self.clist.append(getConfigListEntry("WindowStyleScrollbarTheme", self.TScroll))
			
			self.clist.append(getConfigListEntry(_("----- SCREENS ----------------------------------------------------------------"), ))

			# SCREENS
			if xml.find("screenthemes") is not None:
				# INFOBAR
				InfoBarList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "InfoBar":
						for themes in screens.findall("screentheme"):
							InfoBarList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultinfobar = themes.get("name")

				#print InfoBarList
				if len(InfoBarList) > 0:
					self.SInfobar = ConfigSelection(default=defaultinfobar, choices = InfoBarList)
					config.plugins.MerlinSkinThemes.InfoBar = self.SInfobar
					self.clist.append(getConfigListEntry("InfoBar", self.SInfobar))

				# CHANNELSELECTION
				ChannelSelectionList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "ChannelSelection":
						for themes in screens.findall("screentheme"):
							ChannelSelectionList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultchannelselection = themes.get("name")

				#print ChannelSelectionList
				if len(ChannelSelectionList) > 0:
					self.SChannelSel = ConfigSelection(default=defaultchannelselection, choices = ChannelSelectionList)
					config.plugins.MerlinSkinThemes.ChannelSelection = self.SChannelSel
					self.clist.append(getConfigListEntry("ChannelSelection", self.SChannelSel))

				# MOVIESELECTION
				MovieSelectionList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MovieSelection":
						for themes in screens.findall("screentheme"):
							MovieSelectionList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmovieselection = themes.get("name")

				#print MovieSelectionList
				if len(MovieSelectionList) > 0:
					self.SMovieSel = ConfigSelection(default=defaultmovieselection, choices = MovieSelectionList)
					config.plugins.MerlinSkinThemes.MovieSelection = self.SMovieSel
					self.clist.append(getConfigListEntry("MovieSelection", self.SMovieSel))

				# MOVIEPLAYER
				MoviePlayerList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MoviePlayer":
						for themes in screens.findall("screentheme"):
							MoviePlayerList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmovieplayer = themes.get("name")

				#print MoviePlayerList
				if len(MoviePlayerList) > 0:
					self.SMoviePlay = ConfigSelection(default=defaultmovieplayer, choices = MoviePlayerList)
					config.plugins.MerlinSkinThemes.MoviePlayer = self.SMoviePlay		
					self.clist.append(getConfigListEntry("MoviePlayer", self.SMoviePlay))

				# SECONDINFOBAR
				SecondInfoBarList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "SecondInfoBar":
						for themes in screens.findall("screentheme"):
							SecondInfoBarList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultsecondinfobar = themes.get("name")

				#print SecondInfoBarList
				if len(SecondInfoBarList) > 0:
					self.S2IB = ConfigSelection(default=defaultsecondinfobar, choices = SecondInfoBarList)
					config.plugins.MerlinSkinThemes.SecondInfoBar = self.S2IB
					self.clist.append(getConfigListEntry("SecondInfoBar", self.S2IB))

				# MESSAGEBOX
				MessageBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "MessageBox":
						for themes in screens.findall("screentheme"):
							MessageBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultmessagebox = themes.get("name")

				#print MessageBoxList
				if len(MessageBoxList) > 0:
					self.SMsgBox = ConfigSelection(default=defaultmessagebox, choices = MessageBoxList)
					config.plugins.MerlinSkinThemes.MessageBox = self.SMsgBox
					self.clist.append(getConfigListEntry("MessageBox", self.SMsgBox))

				# INPUTBOX
				InputBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "InputBox":
						for themes in screens.findall("screentheme"):
							InputBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultinputbox = themes.get("name")

				#print InputBoxList
				if len(InputBoxList) > 0:
					self.SInBox = ConfigSelection(default=defaultinputbox, choices = InputBoxList)
					config.plugins.MerlinSkinThemes.InputBox = self.SInBox
					self.clist.append(getConfigListEntry("InputBox", self.SInBox))

				# CHOICEBOX
				ChoiceBoxList = []
				st = xml.find("screenthemes")
				for screens in st.findall("screens"):
					if screens.get("name") == "ChoiceBox":
						for themes in screens.findall("screentheme"):
							ChoiceBoxList.append(themes.get("name"))
							if themes.get("value") == "active":
								defaultchoicebox = themes.get("name")

				#print ChoiceBoxList
				if len(ChoiceBoxList) > 0:
					self.SChoiceBox = ConfigSelection(default=defaultchoicebox, choices = ChoiceBoxList)
					config.plugins.MerlinSkinThemes.ChoiceBox = self.SChoiceBox
					self.clist.append(getConfigListEntry("ChoiceBox", self.SChoiceBox))
					
		except:
			print "[MST] themes.xml in " + MerlinSkinThemes.selSkinName + " corrupt!"
			self.clist.append(getConfigListEntry(_("----- ERROR - themes.xml in " + MerlinSkinThemes.selSkinName + " corrupt! -------"), ))
			
		self["config"].setList(self.clist)
		
	def buttonGreen(self):
		if self.curList == "SkinsList":
			# set new skin
			sel = self["SkinsList"].getCurrent()
			if sel[1][7] == "Default Skin":
				skinfile = "skin.xml"
			else:
				skinfile = "%s/skin.xml" % sel[1][7]

			# Dr. Best Infobar position
			if os.path.isfile("/usr/share/enigma2/merlin_setup.xml") == True:
				config.merlin2.infobar_position_offset_x.value = 0
				config.merlin2.infobar_position_offset_x.save()
				config.merlin2.infobar_position_offset_y.value = 0
				config.merlin2.infobar_position_offset_y.save()
				config.merlin2.movieplayer_infobar_position_offset_x.value = 0
				config.merlin2.movieplayer_infobar_position_offset_x.save()
				config.merlin2.movieplayer_infobar_position_offset_y.value = 0
				config.merlin2.movieplayer_infobar_position_offset_y.save()
				
			config.skin.primary_skin.value = skinfile
			config.skin.primary_skin.save()
			restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("GUI needs a restart to apply a new skin\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
			restartbox.setTitle(_("Restart GUI now?"))
		elif self.curList == "ConfigList":
			askBox = self.session.openWithCallback(self.askYN,MessageBox,_("[apply themes] needs time to build new settings\nDo you want to do this now?"), MessageBox.TYPE_YESNO)
			askBox.setTitle(_("Apply themes now?"))

	def askYN(self, answer):
		if answer is True:
			self.setThemes()
			if SkinName == MerlinSkinThemes.selSkinName:
				restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("GUI needs a restart to apply a new skin\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
				restartbox.setTitle(_("Restart GUI now?"))
			
			else:
				self.session.open(MessageBox, _("Changes to skin " + MerlinSkinThemes.selSkinName + " ready!"), MessageBox.TYPE_INFO)
		#else:
		#	self.exit()
	
	def buttonBlue(self):
		if self.curList == "SkinsList":
			reload(MerlinSkinInfo)
			self.session.open(MerlinSkinInfo.MerlinSkinInfo)
			
		else:
			# check if previewscreen exist
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			rootTheme = curTree.getroot()
			if rootTheme.find("screenthemes") is not None:
				found = False
				themes = rootTheme.find("screenthemes")
				for screens in themes.findall("screens"):
					if screens.get("name") == config.plugins.MerlinSkinThemes.PreviewScreen.value:
						found = True
						
				if found:
					reload(MerlinPreviewScreen)
					self.session.open(MerlinPreviewScreen.MerlinPreviewScreen)
				else:
					self.session.open(MessageBox,_("The screen " + config.plugins.MerlinSkinThemes.PreviewScreen.value + " could found in themes.xml"), MessageBox.TYPE_ERROR, title=_("Error"))
			else:
				self.session.open(MessageBox,_("No section <screenthemes> in themes.xml found"), MessageBox.TYPE_ERROR, title=_("Error"))
				
	def buttonYellow(self):
		if self.curList == "SkinsList":
			if self["SkinsList"].getCurrent()[3][7] == "no themes.xml":
				self.createThemes()

			if self["SkinsList"].getCurrent()[3][7] == "no skin.xml":
				self.delSkinDir()
				
		elif self.curList == "ConfigList":
			if self["config"].getCurrent()[0] == "Design":
				# delete design
				self.deleteDesign()
			
			else:
				# save as design
				self.session.openWithCallback(self.saveDesign, InputBox, title=_("Please enter designname!"))
			
	def saveDesign(self, designname):
		if designname is not None:
			designname = designname.strip()
			
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			xmlroot = curTree.getroot()
			
			if xmlroot.find("designs") is None:
				#print "[MST: designs for this skin NOT found - create]"
				xmldesigns = Tree.SubElement(xmlroot, "designs")
			else:
				#print "[MST: designs for this skin found]"
				xmldesigns = xmlroot.find("designs")

			# check if design exists
			if xmldesigns.find("design[@name='" + designname + "']") is None:
				xmldesign = Tree.SubElement(xmldesigns, "design", {"name": designname, "value": "active"})
				Tree.SubElement(xmldesign, "ColorTheme", {"name": config.plugins.MerlinSkinThemes.ColorTheme.value})
				Tree.SubElement(xmldesign, "FontTheme", {"name": config.plugins.MerlinSkinThemes.FontTheme.value})
				Tree.SubElement(xmldesign, "BorderSetTheme", {"name": config.plugins.MerlinSkinThemes.BorderSetTheme.value})
				Tree.SubElement(xmldesign, "WindowStyleScrollbarTheme", {"name": config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value})
				Tree.SubElement(xmldesign, "InfoBar", {"name": config.plugins.MerlinSkinThemes.InfoBar.value})
				Tree.SubElement(xmldesign, "ChannelSelection", {"name": config.plugins.MerlinSkinThemes.ChannelSelection.value})
				Tree.SubElement(xmldesign, "MovieSelection", {"name": config.plugins.MerlinSkinThemes.MovieSelection.value})
				Tree.SubElement(xmldesign, "MoviePlayer", {"name": config.plugins.MerlinSkinThemes.MoviePlayer.value})
				Tree.SubElement(xmldesign, "SecondInfoBar", {"name": config.plugins.MerlinSkinThemes.SecondInfoBar.value})
				Tree.SubElement(xmldesign, "MessageBox", {"name": config.plugins.MerlinSkinThemes.MessageBox.value})
				Tree.SubElement(xmldesign, "InputBox", {"name": config.plugins.MerlinSkinThemes.InputBox.value})
				Tree.SubElement(xmldesign, "ChoiceBox", {"name": config.plugins.MerlinSkinThemes.ChoiceBox.value})
			
				self.XMLindent(xmlroot, 0)
			
				curTree.write(MerlinSkinThemes.selThemeFile)
				
				self.readThemes()
				
			else: 
				self.session.open(MessageBox,_(designname + " already exists"), MessageBox.TYPE_ERROR)
				
	def deleteDesign(self):
		if self.MSTDesigns.value == "-none-":
			self.session.open(MessageBox,_("nothing to delete"), MessageBox.TYPE_ERROR)
		else:
			curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
			xmlroot = curTree.getroot()
			designs = xmlroot.find("designs")
			for design in designs.findall("design"):
				if design.get("name") == config.plugins.MerlinSkinThemes.Design.value:
					designs.remove(design)
			
					self.XMLindent(xmlroot, 0)

					curTree.write(MerlinSkinThemes.selThemeFile)
		
					self.readThemes()
						
	def setDesign(self):
		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		xmlroot = curTree.getroot()
		designs = xmlroot.find("designs")
		for design in designs.findall("design"):
			if design.get("name") == config.plugins.MerlinSkinThemes.Design.value:
				if design.find("ColorTheme") is not None:
					tmp = design.find("ColorTheme")
					if tmp.get("name") <> "":
						self.TColor.setChoices([self.TColor.getChoices], default=tmp.get("name"))
					
				if design.find("FontTheme") is not None:
					tmp = design.find("FontTheme")
					if tmp.get("name") <> "":
						self.TFont.setChoices([self.TFont.getChoices], default=tmp.get("name"))

				if design.find("BorderSetTheme") is not None:
					tmp = design.find("BorderSetTheme")
					if tmp.get("name") <> "":
						self.TBorder.setChoices([self.TBorder.getChoices], default=tmp.get("name"))
					
				if design.find("WindowStyleScrollbarTheme") is not None:
					tmp = design.find("WindowStyleScrollbarTheme")
					if tmp.get("name") <> "":
						self.TScroll.setChoices([self.TScroll.getChoices], default=tmp.get("name"))
					
				if design.find("InfoBar") is not None:
					tmp = design.find("InfoBar")
					if tmp.get("name") <> "":
						self.SInfobar.setChoices([self.SInfobar.getChoices], default=tmp.get("name"))
					
				if design.find("ChannelSelection") is not None:
					tmp = design.find("ChannelSelection")
					if tmp.get("name") <> "":
						self.SChannelSel.setChoices([self.SChannelSel.getChoices], default=tmp.get("name"))
					
				if design.find("MovieSelection") is not None:
					tmp = design.find("MovieSelection")
					if tmp.get("name") <> "":
						self.SMovieSel.setChoices([self.SMovieSel.getChoices], default=tmp.get("name"))
					
				if design.find("MoviePlayer") is not None:
					tmp = design.find("MoviePlayer")
					if tmp.get("name") <> "":
						self.SMoviePlay.setChoices([self.SMoviePlay.getChoices], default=tmp.get("name"))
					
				if design.find("SecondInfoBar") is not None:
					tmp = design.find("SecondInfoBar")
					if tmp.get("name") <> "":
						self.S2IB.setChoices([self.S2IB.getChoices], default=tmp.get("name"))
					
				if design.find("MessageBox") is not None:
					tmp = design.find("MessageBox")
					if tmp.get("name") <> "":
						self.SMsgBox.setChoices([self.SMsgBox.getChoices], default=tmp.get("name"))
					
				if design.find("InputBox") is not None:
					tmp = design.find("InputBox")
					if tmp.get("name") <> "":
						self.SInBox.setChoices([self.SInBox.getChoices], default=tmp.get("name"))
					
				if design.find("ChoiceBox") is not None:
					tmp = design.find("ChoiceBox")
					if tmp.get("name") <> "":
						self.SChoiceBox.setChoices([self.SChoiceBox.getChoices], default=tmp.get("name"))

				# refresh Screen
				self["config"].hide()
				self["config"].show()
			
	def ok(self):
		if self.curList == "SkinsList":
			if self["SkinsList"].getCurrent()[3][7] == "":
				self.curList = "ConfigList"
				
				self["ListLabel"].setText(_("Configlist"))
				self["DescText"].setText(_("[OK] = switch to skin selection screen\n\n" + 
					"[apply themes] or [preview] takes some time - please wait\n\n" + 
					"[delete design] delete selected design\n\n" + 
					"[save as design] selected themes/screens stored in new design\n\n" +
					"choose Design: -none-\nto reset themes to active settings"
				))

				if self["config"].getCurrent()[0] == "Design":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("delete design"))
					self["key_blue"].setText(_("preview"))
				elif self["config"].getCurrent()[0] == "Skin":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				elif self["config"].getCurrent()[0] == "PreviewScreen":
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				else:
					self["key_green"].setText(_("apply themes"))
					self["key_yellow"].setText(_("save as design"))
					self["key_blue"].setText(_("preview"))
				
				self["key_green"].show()
				self["key_yellow"].show()
				
				self["SkinsList"].hide()
				self["config"].show()
				
				config.plugins.MerlinSkinThemes.Skin.value = self["SkinsList"].getCurrent()[1][7]
			else:
				self.session.open(MessageBox,_("No themes.xml or skin.xml found.\nPlease select a valid skin including themes.xml"), MessageBox.TYPE_ERROR, title=_("Error"))
		else:
			self.curList = "SkinsList"

			self["ListLabel"].setText(_("Skinlist"))
			self["DescText"].setText(_(
				"[OK] = switch to theme config screen for selected skin\n\n" + 
				"[create themes] and [delete] takes some time - please wait\n\n"
			))

			self["key_green"].setText(_("switch to skin"))
			self["key_green"].hide()
			self["key_yellow"].setText(_(""))
			self["key_yellow"].hide()
			self["key_blue"].setText(_("skin info"))

			self.updateSkinsList()
			self["SkinsList"].show()
			self["config"].hide()

	def up(self):
		if self.curList == "SkinsList":
			self[self.curList].up()
		else:
			self["config"].instance.moveSelection(self["config"].instance.moveUp)
			if self["config"].getCurrent()[0] == "Design":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("delete design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "Skin":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "PreviewScreen":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			else:
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
	
	def down(self):
		if self.curList == "SkinsList":
			self[self.curList].down()
		else:
			self["config"].instance.moveSelection(self["config"].instance.moveDown)
			if self["config"].getCurrent()[0] == "Design":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("delete design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "Skin":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			elif self["config"].getCurrent()[0] == "PreviewScreen":
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
			else:
				self["key_green"].setText(_("apply themes"))
				self["key_yellow"].setText(_("save as design"))
				self["key_blue"].setText(_("preview"))
	
	def left(self):
		if self.curList == "SkinsList":
			self[self.curList].pageUp()
		else:
			ConfigListScreen.keyLeft(self)
			if self["config"].getCurrent()[0] == "Design":
				#print "[MST] " + config.plugins.MerlinSkinThemes.Design.value
				if config.plugins.MerlinSkinThemes.Design.value == "-none-":
					self.readThemes()
				else:
					self.setDesign()
	
	def right(self):
		if self.curList == "SkinsList":
			self[self.curList].pageDown()
		else:
			ConfigListScreen.keyRight(self)
			if self["config"].getCurrent()[0] == "Design":
				#print "[MST] " + config.plugins.MerlinSkinThemes.Design.value
				if config.plugins.MerlinSkinThemes.Design.value == "-none-":
					self.readThemes()
				else:
					self.setDesign()
	
	def changedSkinsList(self):
		MerlinSkinThemes.selSkinName = self["SkinsList"].getCurrent()[1][7]
		
		MerlinSkinThemes.selSkinFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/skin.xml"
		MerlinSkinThemes.selThemeFile = resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/themes.xml"
		
		if self["SkinsList"].getCurrent()[2][7] == "active skin":
			self["key_green"].hide()
		else:
			self["key_green"].show()

		if self["SkinsList"].getCurrent()[3][7] == "no skin.xml":
			self["key_green"].hide()

			self["key_yellow"].show()
			self["key_yellow"].setText(_("delete"))
			
		elif self["SkinsList"].getCurrent()[3][7] == "no themes.xml":
			self["key_green"].show()
			
			self["key_yellow"].show()
			self["key_yellow"].setText(_("create themes"))
			
		else:
			self["key_yellow"].show()

			self["key_yellow"].hide()
			self.readThemes()
			
	def updateSkinsList(self):
		self["SkinsList"].buildList()

	def setThemes(self):
		# set all "inactive", set new theme "active"
		#print "[MST] ColorTheme: %s" % config.plugins.MerlinSkinThemes.ColorTheme.value
		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		rootTheme = curTree.getroot()

		curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
		rootSkin = curSkin.getroot()
		
		if rootTheme.find("colortheme") is not None:
			for theme in rootTheme.findall("colortheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.ColorTheme.value:
					theme.set("value", "active")

					# colors
					SkinColors = rootSkin.find("colors")

					# delete old colors
					for color in SkinColors.findall("color"):
						SkinColors.remove(color)
					
					# Set new colors
					ThemeColors = theme.find("colors")
					
					for color in ThemeColors.findall("color"):
						name = color.get("name")
						value = color.get("value")
						
						Tree.SubElement(SkinColors, "color", {"name": name, "value": value})
					
					config.plugins.MerlinSkinThemes.ColorTheme.save()
						
				else:
					theme.set("value", "inactive")

		#print "[MST] FontTheme: %s" % config.plugins.MerlinSkinThemes.FontTheme.value
		if rootTheme.find("fonttheme") is not None:
			for theme in rootTheme.findall("fonttheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.FontTheme.value:
					theme.set("value", "active")

					# fonts
					SkinFonts = rootSkin.find("fonts")

					# delete old fonts
					for font in SkinFonts.findall("font"):
						SkinFonts.remove(font)
					
					# Set new fonts
					ThemeFonts = theme.find("fonts")
					
					for font in ThemeFonts.findall("font"):
						filename = font.get("filename")
						name = font.get("name")
						scale = font.get("scale", "100")
						replacement = font.get("replacement", "0")
						
						Tree.SubElement(SkinFonts, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})
						
					config.plugins.MerlinSkinThemes.FontTheme.save()

				else:
					theme.set("value", "inactive")

		#print "[MST] BorderSetTheme: %s" % config.plugins.MerlinSkinThemes.BorderSetTheme.value
		if rootTheme.find("bordersettheme") is not None:
			for theme in rootTheme.findall("bordersettheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.BorderSetTheme.value:
					theme.set("value", "active")

					# WindowStyle
					for ws in rootSkin.findall("windowstyle"):
						if ws.get("id") == "0":
							for bs in ws.findall("borderset"):
								if bs.get("name") == "bsWindow":
									for px in bs.findall("pixmap"):
										bs.remove(px)
										
									for tbs in theme.findall("borderset"):
										if tbs.get("name") == "bsWindow":
											for tpx in tbs.findall("pixmap"):
												bs.append(Tree.fromstring(Tree.tostring(tpx)))

								if bs.get("name") == "bsListboxEntry":
									for px in bs.findall("pixmap"):
										bs.remove(px)
						
									for tbs in theme.findall("borderset"):
										if tbs.get("name") == "bsListboxEntry":
											for tpx in tbs.findall("pixmap"):
												bs.append(Tree.fromstring(Tree.tostring(tpx)))
						else:
							print "[MST] id not found"
						
					config.plugins.MerlinSkinThemes.BorderSetTheme.save()

				else:
					theme.set("value", "inactive")
					
		#print "[MST] WindowStyleScrollbarTheme: %s" % config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value
		if rootTheme.find("windowstylescrollbartheme") is not None:
			for theme in rootTheme.findall("windowstylescrollbartheme"):
				if theme.get("name") == config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.value:
					theme.set("value", "active")

					# WindowStyleScrollbar
					for wssb in rootSkin.findall("windowstylescrollbar"):
						if wssb.get("id") == "4":
							for all in wssb.findall("*"):
								wssb.remove(all)
										
							for tall in theme.findall("*"):
								wssb.append(Tree.fromstring(Tree.tostring(tall)))

					config.plugins.MerlinSkinThemes.WindowStyleScrollbarTheme.save()

				else:
					theme.set("value", "inactive")
					
		#print "[MST] ScreenTheme - InfoBar: %s" % config.plugins.MerlinSkinThemes.InfoBar.value
		#print "[MST] ScreenTheme - ChannelSelection: %s" % config.plugins.MerlinSkinThemes.ChannelSelection.value
		if rootTheme.find("screenthemes") is not None:
			themes = rootTheme.find("screenthemes")
			for screens in themes.findall("screens"):
				if screens.get("name") == "InfoBar":
					infobars = screens.findall("screentheme")
					for infobar in infobars:
						if infobar.get("name") == config.plugins.MerlinSkinThemes.InfoBar.value:
							infobar.set("value", "active")
							ib = infobar.find("screen")

							# delete old InfoBar screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InfoBar":
									rootSkin.remove(SkinScreen)
							
							# Set new InfoBar screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
						else:
							infobar.set("value", "inactive")
				
				if screens.get("name") == "ChannelSelection":
					for channelselection in screens.findall("screentheme"):
						if channelselection.get("name") == config.plugins.MerlinSkinThemes.ChannelSelection.value:
							channelselection.set("value", "active")
							cs = channelselection.find("screen")

							# delete old ChannelSelection screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "ChannelSelection":
									rootSkin.remove(SkinScreen)
							
							# Set new ChannelSelection screen
							rootSkin.append(Tree.fromstring(Tree.tostring(cs)))
							
							config.plugins.MerlinSkinThemes.ChannelSelection.save()
								
						else:
							channelselection.set("value", "inactive")
				
				if screens.get("name") == "MovieSelection":
					for movieselection in screens.findall("screentheme"):
						if movieselection.get("name") == config.plugins.MerlinSkinThemes.MovieSelection.value:
							movieselection.set("value", "active")
							ms = movieselection.find("screen")

							# delete old MovieSelection screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MovieSelection":
									rootSkin.remove(SkinScreen)
							
							# Set new MovieSelection screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ms)))
								
						else:
							movieselection.set("value", "inactive")
				
				if screens.get("name") == "MoviePlayer":
					for movieplayer in screens.findall("screentheme"):
						if movieplayer.get("name") == config.plugins.MerlinSkinThemes.MoviePlayer.value:
							movieplayer.set("value", "active")
							ms = movieplayer.find("screen")

							# delete old MoviePlayer screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MoviePlayer":
									rootSkin.remove(SkinScreen)
							
							# Set new MoviePlayer screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ms)))
								
						else:
							movieplayer.set("value", "inactive")
				
				if screens.get("name") == "SecondInfoBar":
					for secondinfobar in screens.findall("screentheme"):
						if secondinfobar.get("name") == config.plugins.MerlinSkinThemes.SecondInfoBar.value:
							secondinfobar.set("value", "active")
							sib = secondinfobar.find("screen")

							# delete old SecondInfoBar screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "SecondInfoBar":
									rootSkin.remove(SkinScreen)
							
							# Set new SecondInfoBar screen
							rootSkin.append(Tree.fromstring(Tree.tostring(sib)))
								
							config.plugins.MerlinSkinThemes.SecondInfoBar.save()
								
						else:
							secondinfobar.set("value", "inactive")
				
				if screens.get("name") == "MessageBox":
					for messagebox in screens.findall("screentheme"):
						if messagebox.get("name") == config.plugins.MerlinSkinThemes.MessageBox.value:
							messagebox.set("value", "active")
							mb = messagebox.find("screen")

							# delete old MessageBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "MessageBox":
									rootSkin.remove(SkinScreen)
							
							# Set new MessageBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(mb)))
								
							config.plugins.MerlinSkinThemes.MessageBox.save()
								
						else:
							messagebox.set("value", "inactive")
				
				if screens.get("name") == "InputBox":
					for inputbox in screens.findall("screentheme"):
						if inputbox.get("name") == config.plugins.MerlinSkinThemes.InputBox.value:
							inputbox.set("value", "active")
							ib = inputbox.find("screen")

							# delete old InputBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "InputBox":
									rootSkin.remove(SkinScreen)
							
							# Set new InputBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
							config.plugins.MerlinSkinThemes.InputBox.save()
								
						else:
							inputbox.set("value", "inactive")
				
				if screens.get("name") == "ChoiceBox":
					for choicebox in screens.findall("screentheme"):
						if choicebox.get("name") == config.plugins.MerlinSkinThemes.ChoiceBox.value:
							choicebox.set("value", "active")
							ib = choicebox.find("screen")

							# delete old ChoiceBox screen
							for SkinScreen in rootSkin.findall("screen"):
								if SkinScreen.get("name") == "ChoiceBox":
									rootSkin.remove(SkinScreen)
							
							# Set new ChoiceBox screen
							rootSkin.append(Tree.fromstring(Tree.tostring(ib)))
								
							config.plugins.MerlinSkinThemes.ChoiceBox.save()
								
						else:
							choicebox.set("value", "inactive")
				

		self.XMLindent(rootSkin, 0)
		curSkin.write(MerlinSkinThemes.selSkinFile)
		
		curTree.write(MerlinSkinThemes.selThemeFile)
		
		
	def createThemes(self):
		if os.path.isfile(MerlinSkinThemes.selThemeFile) == False:
			themes = Tree.Element("themes")


			# colors
			colortheme1 = Tree.SubElement(themes, "colortheme", {"name": "orginal", "value": "active"})
			colorsnode1 = Tree.SubElement(colortheme1, "colors")

			colortheme2 = Tree.SubElement(themes, "colortheme", {"name": "orginal - work", "value": "inactive"})
			colorsnode2 = Tree.SubElement(colortheme2, "colors")

			curTree = Tree.parse(MerlinSkinThemes.selSkinFile)
			colors = curTree.find("colors")

			for color in colors.findall("color"):
				name = color.get("name")
				value = color.get("value")
				
				Tree.SubElement(colorsnode1, "color", {"name": name, "value": value})
				Tree.SubElement(colorsnode2, "color", {"name": name, "value": value})


			# fonts
			fonttheme1 = Tree.SubElement(themes, "fonttheme", {"name": "orginal", "value": "active"})
			fontsnode1 = Tree.SubElement(fonttheme1, "fonts")

			fonttheme2 = Tree.SubElement(themes, "fonttheme", {"name": "orginal - work", "value": "inactive"})
			fontsnode2 = Tree.SubElement(fonttheme2, "fonts")

			fonts = curTree.find("fonts")

			for font in fonts.findall("font"):
				filename = font.get("filename")
				name = font.get("name")
				scale = font.get("scale", "100")
				replacement = font.get("replacement", "0")
				
				Tree.SubElement(fontsnode1, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})
				Tree.SubElement(fontsnode2, "font", {"filename": filename, "name": name, "scale": scale, "replacement": replacement})


			# bordersets
			bordersettheme1 = Tree.SubElement(themes, "bordersettheme", {"name": "orginal", "value": "active"})
			bordersettheme2 = Tree.SubElement(themes, "bordersettheme", {"name": "orginal - work", "value": "inactive"})

			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			ws = rootSkin.find("windowstyle")
			for bs in ws.findall("borderstyle"):
				if bs.get("name") == "bsWindow":
					bordersettheme1.append(Tree.fromstring(Tree.tostring(bs)))
					bordersettheme2.append(Tree.fromstring(Tree.tostring(bs)))
			
				if bs.get("name") == "bsListboxEntry":
					bordersettheme1.append(Tree.fromstring(Tree.tostring(bs)))
					bordersettheme2.append(Tree.fromstring(Tree.tostring(bs)))

			
			# windowstylescrollbar
			wssbtheme1 = Tree.SubElement(themes, "windowstylescrollbartheme", {"name": "orginal", "value": "active"})
			wssbtheme2 = Tree.SubElement(themes, "windowstylescrollbartheme", {"name": "orginal - work", "value": "inactive"})
			
			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			for wssb in rootSkin.findall("windowstylescrollbar"):
				if wssb.get("id") == "4":
					for sb in wssb.findall("*"):
						wssbtheme1.append(Tree.fromstring(Tree.tostring(sb)))
						wssbtheme2.append(Tree.fromstring(Tree.tostring(sb)))
			
			
			# screenthemes
			screenthemes = Tree.SubElement(themes, "screenthemes")
			infobar = Tree.SubElement(screenthemes, "screens", {"name": "InfoBar"})
			channelselection = Tree.SubElement(screenthemes, "screens", {"name": "ChannelSelection"})
			movieselection = Tree.SubElement(screenthemes, "screens", {"name": "MovieSelection"})
			movieplayer = Tree.SubElement(screenthemes, "screens", {"name": "MoviePlayer"})
			secondinfobar = Tree.SubElement(screenthemes, "screens", {"name": "SecondInfoBar"})
			messagebox = Tree.SubElement(screenthemes, "screens", {"name": "MessageBox"})
			inputbox = Tree.SubElement(screenthemes, "screens", {"name": "InputBox"})
			choicebox = Tree.SubElement(screenthemes, "screens", {"name": "ChoiceBox"})

			infobar1 = Tree.SubElement(infobar, "screentheme", {"name": "orginal", "value": "active"})
			infobar2 = Tree.SubElement(infobar, "screentheme", {"name": "orginal - work", "value": "inactive"})

			channelselection1 = Tree.SubElement(channelselection, "screentheme", {"name": "orginal", "value": "active"})
			channelselection2 = Tree.SubElement(channelselection, "screentheme", {"name": "orginal - work", "value": "inactive"})

			movieselection1 = Tree.SubElement(movieselection, "screentheme", {"name": "orginal", "value": "active"})
			movieselection2 = Tree.SubElement(movieselection, "screentheme", {"name": "orginal - work", "value": "inactive"})

			movieplayer1 = Tree.SubElement(movieplayer, "screentheme", {"name": "orginal", "value": "active"})
			movieplayer2 = Tree.SubElement(movieplayer, "screentheme", {"name": "orginal - work", "value": "inactive"})

			secondinfobar1 = Tree.SubElement(secondinfobar, "screentheme", {"name": "orginal", "value": "active"})
			secondinfobar2 = Tree.SubElement(secondinfobar, "screentheme", {"name": "orginal - work", "value": "inactive"})

			messagebox1 = Tree.SubElement(messagebox, "screentheme", {"name": "orginal", "value": "active"})
			messagebox2 = Tree.SubElement(messagebox, "screentheme", {"name": "orginal - work", "value": "inactive"})

			inputbox1 = Tree.SubElement(inputbox, "screentheme", {"name": "orginal", "value": "active"})
			inputbox2 = Tree.SubElement(inputbox, "screentheme", {"name": "orginal - work", "value": "inactive"})

			choicebox1 = Tree.SubElement(choicebox, "screentheme", {"name": "orginal", "value": "active"})
			choicebox2 = Tree.SubElement(choicebox, "screentheme", {"name": "orginal - work", "value": "inactive"})
			
			curSkin = Tree.parse(MerlinSkinThemes.selSkinFile)
			rootSkin = curSkin.getroot()
			for SkinScreen in rootSkin.findall("screen"):
				if SkinScreen.get("name") == "InfoBar":
					infobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					infobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "ChannelSelection":
					channelselection1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					channelselection2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MovieSelection":
					movieselection1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					movieselection2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MoviePlayer":
					movieplayer1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					movieplayer2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					
				if SkinScreen.get("name") == "SecondInfoBar":
					secondinfobar1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					secondinfobar2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "MessageBox":
					messagebox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					messagebox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "InputBox":
					inputbox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					inputbox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
				if SkinScreen.get("name") == "ChoiceBox":
					choicebox1.append(Tree.fromstring(Tree.tostring(SkinScreen)))
					choicebox2.append(Tree.fromstring(Tree.tostring(SkinScreen)))
			
					
			# Sort
			self.XMLindent(themes, 0)
			
			# save xml
			themexml = open(MerlinSkinThemes.selThemeFile, "w")
			themexml.write(Tree.tostring(themes))
			themexml.close()
			
			self.updateSkinsList()
	
	def delSkinDir(self):
		print "[MST] Delete: %s" % resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/"
		shutil.rmtree(resolveFilename(SCOPE_SKIN) + MerlinSkinThemes.selSkinName + "/")
		self.updateSkinsList()
		
	def restartGUI(self, answer):
		if answer is True:
			#self.setTheme()
			self.session.open(TryQuitMainloop, 3)
		else:
			self.exit()
	
	def save(self):
		restartbox = self.session.openWithCallback(self.restartGUI,MessageBox,_("For apply new skin settings.\nDo you want to Restart the GUI now?"), MessageBox.TYPE_YESNO)
		restartbox.setTitle(_("Restart GUI now?"))

	def exit(self):
		print '[MST] closing'
		self["SkinsList"].onSelectionChanged.remove(self.changedSkinsList)
		self.close()
		
	def XMLindent(self, elem, level):
		i = "\n" + (level*"    ")
		#a = "\n%%-%ds" % level
		#i = a % '  '
		
		if len(elem):
			if not elem.text or not elem.text.strip():
				elem.text = i + "    "
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
			for elem in elem:
				self.XMLindent(elem, level+1)
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
		else:
			if level and (not elem.tail or not elem.tail.strip()):
				elem.tail = i
			
def main(session, **kwargs):
	session.open(MerlinSkinThemes)

def Plugins(path,**kwargs):
	list = [PluginDescriptor(name = "MerlinSkinThemes", description = "MerlinSkinThemes", where = PluginDescriptor.WHERE_PLUGINMENU, icon = "plugin.png", fnc = main)]
	return list		

# =================================================================================================

class GetSkinsList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setFont(1, gFont("Regular", 17))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		dirs = os.listdir(resolveFilename(SCOPE_SKIN))
		for dir in dirs:
			if os.path.isdir(resolveFilename(SCOPE_SKIN) + dir) is True:
				curSkinFile = resolveFilename(SCOPE_SKIN) + dir + "/skin.xml"
				curThemeFile = resolveFilename(SCOPE_SKIN) + dir + "/themes.xml"
				
				info = ""
				status = ""

				if os.path.exists(curSkinFile) or os.path.exists(curThemeFile):
					if os.path.exists(curSkinFile) is False:
						info = "no skin.xml"

					if os.path.exists(curThemeFile) is False:
						info = "no themes.xml"

					if dir == SkinName:
						status = "active skin"

					res = [
						dir,
						(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 250, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, dir),
						(eListboxPythonMultiContent.TYPE_TEXT, 255, 0, 155, 24, 1, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
						(eListboxPythonMultiContent.TYPE_TEXT, 415, 0, 155, 24, 1, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, info),
					]
					list.append(res)
				
		self.list = list
		self.l.setList(list)
		
# =================================================================================================

class GetColorThemesList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		for theme in curTree.findall("colortheme"):
			status = ""
			if theme.attrib["value"] == "active":
				status = theme.attrib["value"]
				
			res = [
				theme,
				(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 245, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, theme.attrib["name"]),
				(eListboxPythonMultiContent.TYPE_TEXT, 250, 0, 225, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
				(eListboxPythonMultiContent.TYPE_TEXT, 480, 0, 20, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, ""),
			]
			list.append(res)

		self.list = list
		self.l.setList(list)

# =================================================================================================

class GetFontThemesList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		curTree = Tree.parse(MerlinSkinThemes.selThemeFile)
		for theme in curTree.findall("fonttheme"):
			status = ""
			if theme.attrib["value"] == "active":
				status = theme.attrib["value"]
				
			res = [
				theme,
				(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 245, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, theme.attrib["name"]),
				(eListboxPythonMultiContent.TYPE_TEXT, 250, 0, 225, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, status),
				(eListboxPythonMultiContent.TYPE_TEXT, 480, 0, 20, 24, 0, RT_HALIGN_RIGHT|RT_VALIGN_CENTER, ""),
			]
			list.append(res)

		self.list = list
		self.l.setList(list)

# =================================================================================================

class GetScreensList(MenuList, MerlinSkinThemes):
	def __init__(self, list, enableWrapAround = True):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(26)
		
	def buildList(self):
		list = []

		#print "[MST] SkinFile: " + SkinFile
		#curTree = Tree.parse(SkinFile)
		#for screens in curTree.findall("screen"):
		#	res = [
		#		screens,
		#		(eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, screens.attrib["name"]),
		#	]
		#	list.append(res)

		res = ["", (eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, "InfoBar"),]
		list.append(res)
		
		res = ["", (eListboxPythonMultiContent.TYPE_TEXT, 5, 0, 500, 24, 0, RT_HALIGN_LEFT|RT_VALIGN_CENTER, "ChannelSelection"),]
		list.append(res)

		self.list = list
		self.l.setList(list)
	